/*
 * taskview-io-proxy.c
 *
 * TaskviewIOProxy
 *
 * Copyright © 2010 Salomon Sickert
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the licence or (at
 * your option) any later version.
 *
 * See the included COPYING file for more information.
 *
 * Authors:
 *    Salomon Sickert <sickert@in.tum.de>
 */

#include "taskview.h"
#include "taskview-io-proxy.h"

#define TASKVIEW_IO_PROXY_GET_PRIVATE(obj)	(G_TYPE_INSTANCE_GET_PRIVATE ((obj), TASKVIEW_TYPE_IO_PROXY, TaskviewIOProxyPrivate))

G_DEFINE_TYPE (TaskviewIOProxy, taskview_io_proxy, TASKVIEW_TYPE_GENERIC_PROXY);

struct _TaskviewIOProxyPrivate
{
  GDBusProxy *proxy;
};

/* ----------------------------------------------------------------------------------------------------
 * GObject Properties
 * ---------------------------------------------------------------------------------------------------- */

enum /* properties */
{
  PROP_0,

  /* Properties of the remote object */
  PROP_TYPE,

  PROP_CURRENT_ITEM,
  PROP_DESTINATION_URI,
  PROP_SOURCE_URI,

  PROP_THROUGHPUT_SIZE,
  PROP_PROCESSED_SIZE,
  PROP_TOTAL_SIZE,

  PROP_THROUGHPUT_ITEMS,
  PROP_PROCESSED_ITEMS,
  PROP_TOTAL_ITEMS
};

static void
taskview_io_proxy_get_property (GObject    *object,
                                guint       property_id,
                                GValue     *value,
                                GParamSpec *pspec)
{
  TaskviewIOProxy *self = TASKVIEW_IO_PROXY (object);
  GVariant *result = NULL;
  const gchar* type = NULL;
  const gchar* name = NULL;

  switch (property_id)
    {
      /* Properties of the remote object, FIXME: Quark? */
    case PROP_TYPE:
      name = "Type";
      break;

    case PROP_CURRENT_ITEM:
      name = "CurrentItem";
      break;

    case PROP_DESTINATION_URI:
      name = "DestinationURI";
      break;

    case PROP_SOURCE_URI:
      name = "SourceURI";
      break;

    case PROP_THROUGHPUT_SIZE:
      name = "ThroughputSize";
      break;

    case PROP_PROCESSED_SIZE:
      name = "ProcessedSize";
      break;

    case PROP_TOTAL_SIZE:
      name = "TotalSize";
      break;

    case PROP_THROUGHPUT_ITEMS:
      name = "ThroughputItems";
      break;

    case PROP_PROCESSED_ITEMS:
      name = "ProcessedItems";
      break;

    case PROP_TOTAL_ITEMS:
      name = "TotalItems";
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
      break;
    }

  if (self->priv->proxy != NULL && name != NULL)
    {
      result = g_dbus_proxy_get_cached_property (self->priv->proxy, name);
    }

  if (result != NULL)
    {
      type = g_variant_get_type_string (result);

      if (g_strcmp0 (type, "t") == 0)
        g_value_set_uint64 (value, g_variant_get_uint64 (result));
      else if (g_strcmp0 (type, "i") == 0)
        g_value_set_int (value, g_variant_get_int32 (result));
      else if (g_strcmp0 (type, "u") == 0)
        g_value_set_uint (value, g_variant_get_uint32 (result));
      else if (g_strcmp0 (type, "s") == 0)
        g_value_set_string (value, g_variant_get_string (result, NULL));
      else if (g_strcmp0 (type, "as") == 0)
        g_value_set_boxed (value, g_variant_get_strv (result, NULL));
      else
        g_warn_if_reached ();

      g_variant_unref (result);
    }
}

/* ----------------------------------------------------------------------------------------------------
 * Signal Handler
 * ---------------------------------------------------------------------------------------------------- */

static void
on_properties_changed (GDBusProxy          *proxy,
                       GVariant            *changed_properties,
                       const gchar* const  *invalidated_properties,
                       gpointer             user_data)
{
  TaskviewIOProxy *self = TASKVIEW_IO_PROXY (user_data);

  if (g_variant_n_children (changed_properties) > 0)
    {
      GVariantIter iter;
      GVariant *value;
      gchar *key;

      g_variant_iter_init (&iter, changed_properties);

      while (g_variant_iter_loop (&iter, "{&sv}", &key, &value))
        {
          if (g_strcmp0 (key, "Type") == 0)
            g_object_notify (G_OBJECT (self), "type");
          else if (g_strcmp0 (key, "CurrentItem") == 0)
            g_object_notify (G_OBJECT (self), "current-item");
          else if (g_strcmp0 (key, "DestinationURI") == 0)
            g_object_notify (G_OBJECT (self), "destination-uri");
          else if (g_strcmp0 (key, "SourceURI") == 0)
            g_object_notify (G_OBJECT (self), "source-uri");
          else if (g_strcmp0 (key, "ThroughputSize") == 0)
            g_object_notify (G_OBJECT (self), "throughput-size");
          else if (g_strcmp0 (key, "ProcessedSize") == 0)
            g_object_notify (G_OBJECT (self), "processed-size");
          else if (g_strcmp0 (key, "TotalSize") == 0)
            g_object_notify (G_OBJECT (self), "total-size");
          else if (g_strcmp0 (key, "ThroughputItems") == 0)
            g_object_notify (G_OBJECT (self), "throughput-items");
          else if (g_strcmp0 (key, "ProcessedItems") == 0)
            g_object_notify (G_OBJECT (self), "processed-items");
          else if (g_strcmp0 (key, "TotalItems") == 0)
            g_object_notify (G_OBJECT (self), "total-items");
          else
            g_debug ("Unknown Property changed: %s", key);
        }
    }
}

static void
on_proxy_appeared (GObject *source_object, GAsyncResult *res, gpointer user_data)
{
  TaskviewIOProxy *self = TASKVIEW_IO_PROXY (user_data);
  GDBusProxy *proxy;
  GError *error = NULL;

  proxy =  g_dbus_proxy_new_for_bus_finish (res, &error);

  if (proxy == NULL)
    {
      g_printerr ("Error creating proxy: %s\n", error->message);
      g_error_free (error);
      return;
    }

  g_signal_connect (proxy,
                    "g-properties-changed",
                    G_CALLBACK (on_properties_changed),
                    self);

  self->priv->proxy = proxy;

  g_object_notify (G_OBJECT (self), "type");
  g_object_notify (G_OBJECT (self), "current-item");
  g_object_notify (G_OBJECT (self), "destination-uri");
  g_object_notify (G_OBJECT (self), "source-uri");
  g_object_notify (G_OBJECT (self), "processed-size");
  g_object_notify (G_OBJECT (self), "total-size");
  g_object_notify (G_OBJECT (self), "throughput-items");
  g_object_notify (G_OBJECT (self), "processed-items");
  g_object_notify (G_OBJECT (self), "total-items");
}

/* ----------------------------------------------------------------------------------------------------
 * GObject
 * ---------------------------------------------------------------------------------------------------- */

static void
taskview_io_proxy_constructed (GObject *gobject)
{
  TaskviewIOProxy *self = TASKVIEW_IO_PROXY (gobject);
  GDBusConnection *connection;
  gchar *dbus_name = NULL;
  gchar *object_path = NULL;

  g_object_get (&self->parent,
                "dbus-name", &dbus_name,
                "dbus-object-path", &object_path,
                "dbus-connection", &connection,
                NULL);

  g_dbus_proxy_new (connection,
                    G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START,
                    NULL, /* FIXME: Add org.gnome.TaskView.? */
                    dbus_name,
                    object_path,
                    TASKVIEW_IO_INTERFACE_NAME,
                    NULL,
                    on_proxy_appeared,
                    self);

  g_object_unref (connection);
  g_free (dbus_name);
  g_free (object_path);

	G_OBJECT_CLASS (taskview_io_proxy_parent_class)->constructed (gobject);
}

static void
taskview_io_proxy_dispose (GObject *gobject)
{
  TaskviewIOProxy *self = TASKVIEW_IO_PROXY (gobject);

  if (self->priv->proxy != NULL)
    {
      g_object_unref (self->priv->proxy);
      self->priv->proxy = NULL;
    }

  G_OBJECT_CLASS (taskview_io_proxy_parent_class)->dispose (gobject);
}

static void
taskview_io_proxy_class_init (TaskviewIOProxyClass *klass)
{
  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

  g_type_class_add_private (gobject_class, sizeof (TaskviewIOProxyPrivate));

  gobject_class->constructed = taskview_io_proxy_constructed;
  gobject_class->dispose = taskview_io_proxy_dispose;
  gobject_class->get_property = taskview_io_proxy_get_property;

  GParamSpec *pspec;

  /* Properties of the remote object */

  /* Type */
  pspec = g_param_spec_int  ("type",
                             "Type",
                             "The type of the IO transfer",
                             0,
                             TASKVIEW_IO_UNKNOWN,
                             TASKVIEW_IO_UNKNOWN,
                             G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_TYPE, pspec);

  /* CurrentItem */
  pspec = g_param_spec_uint ("current-item",
                             "CurrentItem",
                             "The index of the current processed item, contained in DestinationURI",
                             0,
                             G_MAXUINT32,
                             G_MAXUINT32,
                             G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_CURRENT_ITEM, pspec);

  /* DestinationURI */
  pspec = g_param_spec_boxed ("destination-uri",
                              "DestinationURI",
                              "All destination uris",
                              G_TYPE_STRV,
                              G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_DESTINATION_URI, pspec);

  /* SourceURI */
  pspec = g_param_spec_boxed ("source-uri",
                              "SourceURI",
                              "All source uris",
                              G_TYPE_STRV,
                              G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_SOURCE_URI, pspec);

  /* ThroughputSize */
  pspec = g_param_spec_uint64  ("throughput-size",
                                "ThroughputSize",
                                "The size throughput of the Task",
                                0,
                                G_MAXUINT64,
                                G_MAXUINT64,
                                G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_THROUGHPUT_SIZE, pspec);

  /* ProcessedSize */
  pspec = g_param_spec_uint64  ("processed-size",
                                "ProcessedSize",
                                "The processed size of the Task",
                                0,
                                G_MAXUINT64,
                                G_MAXUINT64,
                                G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_PROCESSED_SIZE, pspec);

  /* TotalSize */
  pspec = g_param_spec_uint64  ("total-size",
                                "TotalSize",
                                "The total size of the Task",
                                0,
                                G_MAXUINT64,
                                G_MAXUINT64,
                                G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_TOTAL_SIZE, pspec);

  /* ThroughputItems */
  pspec = g_param_spec_uint64  ("throughput-items",
                                "ThroughputItems",
                                "The items throughput of the Task",
                                0,
                                G_MAXUINT64,
                                G_MAXUINT64,
                                G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_THROUGHPUT_ITEMS, pspec);

  /* ProcessedItems */
  pspec = g_param_spec_uint64  ("processed-items",
                                "ProcessedItems",
                                "The number processed items of the Task",
                                0,
                                G_MAXUINT64,
                                G_MAXUINT64,
                                G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_PROCESSED_ITEMS, pspec);

  /* TotalItems */
  pspec = g_param_spec_uint64  ("total-items",
                                "TotalFiles",
                                "The number of all items of the Task",
                                0,
                                G_MAXUINT64,
                                G_MAXUINT64,
                                G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_TOTAL_ITEMS, pspec);
}

static void
taskview_io_proxy_init (TaskviewIOProxy *self)
{
  self->priv = TASKVIEW_IO_PROXY_GET_PRIVATE (self);
  self->priv->proxy = NULL;
}

/* ----------------------------------------------------------------------------------------------------
 * C Wrapper for g_object_get
 * ---------------------------------------------------------------------------------------------------- */

gint
taskview_io_proxy_get_io_type (TaskviewIOProxy *self)
{
  gint value;
  g_object_get (self, "type", &value, NULL);
  return value;
}

guint
taskview_io_proxy_get_current_item (TaskviewIOProxy *self)
{
  guint value;
  g_object_get (self, "current-item", &value, NULL);
  return value;
}

GStrv
taskview_io_proxy_get_destination_uri (TaskviewIOProxy *self)
{
  GStrv value;
  g_object_get (self, "destination-uri", &value, NULL);
  return value;
}

GStrv
taskview_io_proxy_get_source_uri (TaskviewIOProxy *self)
{
  GStrv value;
  g_object_get (self, "source-uri", &value, NULL);
  return value;
}

guint64
taskview_io_proxy_get_throughput_size (TaskviewIOProxy *self)
{
  guint64 value;
  g_object_get (self, "throughput-size", &value, NULL);
  return value;
}

guint64
taskview_io_proxy_get_total_size (TaskviewIOProxy *self)
{
  guint64 value;
  g_object_get (self, "total-size", &value, NULL);
  return value;
}

guint64
taskview_io_proxy_get_processed_size (TaskviewIOProxy *self)
{
  guint64 value;
  g_object_get (self, "processed-size", &value, NULL);
  return value;
}

guint64
taskview_io_proxy_get_throughput_items (TaskviewIOProxy *self)
{
  guint64 value;
  g_object_get (self, "throughput-items", &value, NULL);
  return value;
}

guint64
taskview_io_proxy_get_total_items (TaskviewIOProxy *self)
{
  guint64 value;
  g_object_get (self, "total-items", &value, NULL);
  return value;
}

guint64
taskview_io_proxy_get_processed_items (TaskviewIOProxy *self)
{
  guint64 value;
  g_object_get (self, "processed-items", &value, NULL);
  return value;
}
